@@ -6,10 +6,12 @@ from .help import HelpScreen
 from .mcp_install_wizard import MCPInstallWizardScreen
 from .settings import SettingsScreen
 from .tools import ToolsScreen
+from .autosave_picker import AutosavePicker
 
 __all__ = [
     "HelpScreen",
     "SettingsScreen",
     "ToolsScreen",
     "MCPInstallWizardScreen",
+    "AutosavePicker",
 ]
